﻿// Sequencer.cs (c) Charles Petzold, 2009

using System;
using System.Collections.Generic;

namespace Petzold.SoundSynthesis
{
    public class Sequencer : ISampleProvider
    {
        int tempo = 360;
        int samplesPerQuarter = 44100 / 6;
        int sampleCounter = 0;

        int accumulatedSeconds = 0;
        int accumulatedSampleTicks = 0;
        int positionIndex;

        public event SequencerPositionChangedEventHandler IndexChanged;

        public Oscillator Oscillator { get; set; }
        public IList<Pitch> Pitches { protected set; get; }

        public int Tempo
        {
            set
            {
                tempo = value;
                samplesPerQuarter = 44100 * 60 / tempo;
            }
            get
            {
                return tempo;
            }
        }

        public Sequencer()
        {
            Pitches = new List<Pitch>();
            positionIndex = -1;
        }

        public short GetNextSample()
        {
            sampleCounter++;

            if (sampleCounter > samplesPerQuarter)
            {
                positionIndex++;

                if (positionIndex >= Pitches.Count)
                    positionIndex = 0;

                if (IndexChanged != null)
                    IndexChanged(this, new SequencerPositionChangedEventArgs(positionIndex, 
                                                TimeSpan.FromSeconds(accumulatedSeconds + accumulatedSampleTicks / 44100.0)));

                Oscillator.Frequency = Pitches[positionIndex].Frequency;

                sampleCounter = 0;
            }

            // Bump up the counters for how much time has elapsed
            if (++accumulatedSampleTicks == 44100)
            {
                accumulatedSeconds++;
                accumulatedSampleTicks = 0;
            }

            return Oscillator.GetNextSample();
        }
    }
}
