﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;
using System.IO;
using Petzold.SoundSynthesis;

namespace SimpleSequencer
{
    public partial class MainPage : UserControl
    {
        Oscillator osc;
        Sequencer seq;
        Attenuator att;
        List<SequencerPositionChangedEventArgs> timedPositions = new List<SequencerPositionChangedEventArgs>();
        object timedPositionLocker = new object();
        DispatcherTimer tmr;
    
        public MainPage()
        {
            osc = new Oscillator();

            seq = new Sequencer();
            seq.Oscillator = osc;

            att = new Attenuator();
            att.SampleProvider = seq;

            seq.Pitches.Add(new Pitch(Note.C, 4));
            seq.Pitches.Add(new Pitch(Note.E, 4));
            seq.Pitches.Add(new Pitch(Note.G, 4));
            seq.Pitches.Add(new Pitch(Note.C, 5));
            seq.Pitches.Add(new Pitch(Note.G, 4));
            seq.Pitches.Add(new Pitch(Note.E, 4));
            seq.Pitches.Add(new Pitch(Note.C, 4));
            seq.Pitches.Add(new Pitch(Note.G, 3));
            seq.Pitches.Add(new Pitch(Note.E, 3));
            seq.Pitches.Add(new Pitch(Note.C, 3));
            seq.Pitches.Add(new Pitch(Note.E, 3));
            seq.Pitches.Add(new Pitch(Note.G, 3));
            seq.Pitches.Add(new Pitch(Note.C, 4));
            seq.Pitches.Add(new Pitch(Note.E, 4));
            seq.Pitches.Add(new Pitch(Note.G, 4));
            seq.Pitches.Add(new Pitch(Note.E, 4));

            InitializeComponent();

            seq.IndexChanged += OnSequencerIndexChanged;
            staff.Pitches = seq.Pitches;

            PcmMediaStreamSource streamSource = new PcmMediaStreamSource(44100, 1);
            streamSource.SampleProvider = att;
            mediaElement.SetSource(streamSource);

            tmr = new DispatcherTimer();
            tmr.Interval = TimeSpan.FromMilliseconds(10);
            tmr.Tick += new EventHandler(OnTimerTick);
        }

        void OnPlayPauseCheckBoxChecked(object sender, RoutedEventArgs args)
        {
            CheckBox chkbox = sender as CheckBox;

            if ((bool)chkbox.IsChecked)
            {
                mediaElement.Play();
                tmr.Start();
            }
            else
            {
                mediaElement.Pause();
                tmr.Stop();
            }
        }

        void OnWaveformRadioChecked(object sender, RoutedEventArgs args)
        {
            RadioButton radio = sender as RadioButton;
            Waveform waveform = (Waveform)Enum.Parse(typeof(Waveform), radio.Tag as string, true);
            osc.Waveform = waveform;
        }

        // Event handler called asynchronously from Sequencer
        void OnSequencerIndexChanged(object sender, SequencerPositionChangedEventArgs args)
        {
            lock (timedPositionLocker)
            {
                timedPositions.Add(args);
            }
        }

        void OnTimerTick(object sender, EventArgs e)
        {
            TimeSpan mediaElementElapsedTime = mediaElement.Position;

            lock (timedPositionLocker)
            {
                for (int index = 0; index < timedPositions.Count; index++)
                {
                    SequencerPositionChangedEventArgs args = timedPositions[index];

                    if (mediaElementElapsedTime >= args.ElapsedTime)
                    {
                        staff.PositionIndex = args.PositionIndex;
                        timedPositions.RemoveAt(index);
                        break;
                    }
                }
            }
        }

        void OnTempoScrollValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            seq.Tempo = (int)args.NewValue;
        }

        void OnVolumeScrollValueChanged(object sender, RoutedPropertyChangedEventArgs<double> args)
        {
            att.Attenuation = (int)args.NewValue;
        }
    }
}
