﻿using System;
using System.Collections.Generic;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;
using Petzold.SoundSynthesis;

namespace SimpleSequencer
{
    public partial class Staff : UserControl
    {
        IList<Pitch> pitches;
        int positionIndex;
        Path bouncingBall;

        QuarterNote quarterDragging;
        double draggingInitialY;
        double draggingInitialMouseY;

        Pitch[] positionPitches = 
                        { new Pitch(Note.C, 6), new Pitch(Note.B, 5), new Pitch(Note.A, 5), new Pitch(Note.G, 5),
                          new Pitch(Note.F, 5), new Pitch(Note.E, 5), new Pitch(Note.D, 5),
                          new Pitch(Note.C, 5), new Pitch(Note.B, 4), new Pitch(Note.A, 4), new Pitch(Note.G, 4),
                          new Pitch(Note.F, 4), new Pitch(Note.E, 4), new Pitch(Note.D, 4), new Pitch(Note.C, 4),

                          new Pitch(Note.C, 4), new Pitch(Note.B, 3), new Pitch(Note.A, 3), new Pitch(Note.G, 3),
                          new Pitch(Note.F, 3), new Pitch(Note.E, 3), new Pitch(Note.D, 3),
                          new Pitch(Note.C, 3), new Pitch(Note.B, 2), new Pitch(Note.A, 2), new Pitch(Note.G, 2),
                          new Pitch(Note.F, 2), new Pitch(Note.E, 2), new Pitch(Note.D, 2), new Pitch(Note.C, 2) };

        double[] xPositions = new double[16];
        double[] yPositions = new double[30];

        public Staff()
        {
            InitializeComponent();
            Brush blackBrush = new SolidColorBrush(Colors.Black);

            for (int xPositionIndex = 0; xPositionIndex < xPositions.Length; xPositionIndex++)
            {
                xPositions[xPositionIndex] = 96 + 48 * xPositionIndex;
            }

            for (int yPositionIndex = 0; yPositionIndex < yPositions.Length; yPositionIndex++)
            {
                double y = 6 * (8 + yPositionIndex);

                if (yPositionIndex > 14)
                {
                    y += 48;
                }

                yPositions[yPositionIndex] = y;
            }

            for (int yPositionIndex = 0; yPositionIndex < 30; yPositionIndex++)
            {
                if (yPositionIndex <= 14 && yPositionIndex % 2 == 1)
                    continue;

                if (yPositionIndex > 14 && yPositionIndex % 2 == 0)
                    continue;

                double y = yPositions[yPositionIndex];

                if (yPositionIndex < 4 || yPositionIndex == 14 || yPositionIndex == 15 || yPositionIndex > 25)
                {
                    for (int xPositionIndex = 0; xPositionIndex < 16; xPositionIndex++)
                    {
                        double x = xPositions[xPositionIndex];

                        Line line = new Line();
                        line.Stroke = blackBrush;
                        line.X1 = x - 12;
                        line.Y1 = y;
                        line.X2 = x + 12;
                        line.Y2 = y;
                        LayoutRoot.Children.Add(line);
                    }
                }
                else
                {
                    Line line = new Line();
                    line.Stroke = blackBrush;
                    line.X1 = 12;
                    line.Y1 = y;
                    line.X2 = xPositions[15] + 48;
                    line.Y2 = y;
                    LayoutRoot.Children.Add(line);
                }
            }

            // Draw vertical lines
            double xVertical = 12;

            for (int i = 0; i < 2; i++)
            {
                Line line = new Line();
                line.Stroke = blackBrush;
                line.X1 = xVertical;
                line.Y1 = yPositions[4];
                line.X2 = xVertical;
                line.Y2 = yPositions[25];
                canvas.Children.Add(line);

                xVertical = xPositions[15] + 48;
            }

            // Draw repetition dots
            int[] ySpotIndices = { 7, 9, 20, 22 };

            for (int i = 0; i < 4; i++)
            {
                EllipseGeometry ellipse = new EllipseGeometry();
                ellipse.RadiusX = 3;
                ellipse.RadiusY = 3;
                ellipse.Center = new Point(xVertical - 6, yPositions[ySpotIndices[i]]);

                Path path = new Path();
                path.Fill = blackBrush;
                path.Data = ellipse;
                canvas.Children.Add(path);
            }

            // Draw bouncing ball
            EllipseGeometry ballEllipse = new EllipseGeometry();
            ballEllipse.RadiusX = 3;
            ballEllipse.RadiusY = 3;

            bouncingBall = new Path();
            bouncingBall.Fill = new SolidColorBrush(Colors.Red);
            bouncingBall.Data = ballEllipse;

            Canvas.SetLeft(bouncingBall, xPositions[0]);
            Canvas.SetTop(bouncingBall, (yPositions[14] + yPositions[15]) / 2);

            canvas.Children.Add(bouncingBall);
        }

        public IList<Pitch> Pitches 
        {
            get { return pitches; }
            set
            {
                pitches = value;

                for (int xPositionIndex = 0; xPositionIndex < pitches.Count; xPositionIndex++)
                {
                    QuarterNote quarter = new QuarterNote();
                    quarter.Index = xPositionIndex;
                    Canvas.SetLeft(quarter, xPositions[xPositionIndex]);

                    for (int yPositionIndex = 0; yPositionIndex < positionPitches.Length; yPositionIndex++)
                    {
                        if (pitches[xPositionIndex].Note == positionPitches[yPositionIndex].Note && 
                            pitches[xPositionIndex].Octave == positionPitches[yPositionIndex].Octave)
                        {
                            Canvas.SetTop(quarter, yPositions[yPositionIndex]); 
                            break;
                        }
                    }

                    canvas.Children.Add(quarter);
                }
            }
        }

        public int PositionIndex
        {
            set
            {
                positionIndex = value;
                Canvas.SetLeft(bouncingBall, xPositions[positionIndex]);
            }
            get
            {
                return positionIndex;
            }
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs e)
        {
            QuarterNote quarter = FindParent(e.OriginalSource as DependencyObject, typeof(QuarterNote)) as QuarterNote;

            if (quarter == null)
                return;

            if ((Keyboard.Modifiers & ModifierKeys.Shift) == ModifierKeys.Shift)
            {
                int previousSharpOrFlat = quarter.SharpOrFlat;
                int sharpOrFlat = previousSharpOrFlat + 1;

                if (sharpOrFlat == 2)
                    sharpOrFlat = -1;

                quarter.SharpOrFlat = sharpOrFlat;

                Pitch previousPitch = pitches[quarter.Index];
                Note newNote = previousPitch.Note + (sharpOrFlat - previousSharpOrFlat);
                pitches[quarter.Index] = new Pitch(newNote, previousPitch.Octave);

                return;
            }

            quarterDragging = quarter;
            draggingInitialY = Canvas.GetTop(quarterDragging);
            draggingInitialMouseY = e.GetPosition(this).Y;
            CaptureMouse();

            base.OnMouseLeftButtonDown(e);
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            if (quarterDragging != null)
            {
                Point mousePoint = e.GetPosition(this);
                double y = draggingInitialY + mousePoint.Y - draggingInitialMouseY;
                y = Math.Max(yPositions[0], Math.Min(yPositions[yPositions.Length - 1], y));
                Canvas.SetTop(quarterDragging, y);

            }

            base.OnMouseMove(e);
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs e)
        {
            if (quarterDragging != null)
            {
                Point mousePoint = e.GetPosition(this);
                double y = draggingInitialY + mousePoint.Y - draggingInitialMouseY;
                double minDistance = Double.PositiveInfinity;
                int index = 0;

                for (int i = 0; i < yPositions.Length; i++)
                {
                    double distance = Math.Abs(y - yPositions[i]);

                    if (distance < minDistance)
                    {
                        index = i;
                        minDistance = distance;
                    }
                }

                Canvas.SetTop(quarterDragging, yPositions[index]);

                Pitch basePitch = positionPitches[index];
                Pitch newPitch = new Pitch(basePitch.Note + quarterDragging.SharpOrFlat, basePitch.Octave);
                Pitches[quarterDragging.Index] = newPitch;

                quarterDragging = null;
                ReleaseMouseCapture();
            }

            base.OnMouseLeftButtonUp(e);
        }


        // On Lost Mouse Capture



        DependencyObject FindParent(DependencyObject source, Type parentType)
        {
            if (source == null)
                return null;

            if (source.GetType() == parentType)
                return source;

            return FindParent(VisualTreeHelper.GetParent(source), parentType);
        }
    }
}
